﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using PeerWcfComm;

namespace LoggerPeerApplication
{
    /// <summary>
    /// The logging peer.  Participates in IWorker and ISpectator communications.
    /// It simply logs to standard output.
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class Logger : IWorker, ISpectator
    {
        private ServiceHost _host;
        private Dictionary<Guid, WorkerStatus> _workers;

        public Logger()
        {
            _workers = new Dictionary<Guid, WorkerStatus>();
        }

        public void Start()
        {
            _host = new ServiceHost(this);
            _host.Open();
        }

        public void Stop()
        {
            _host.Close();
        }

        #region IWorker Members

        void IWorker.Heartbeat(WorkerStatus status)
        {
            //Only log a change in state.
            bool changed = false;

            lock (_workers)
            {
                if (_workers.ContainsKey(status.WorkerId))
                {
                    WorkerStatus s = _workers[status.WorkerId];

                    if (s.State != status.State)
                        changed = true;
                }
                else
                    changed = true;

                _workers[status.WorkerId] = status;
            }

            if (changed)
                LogWorkerStatusChange(status);
        }

        void IWorker.DoWork(Guid workerId, WorkItem work)
        {
            //Ignored. 
            //This can be assumed by the WorkItemState.Queued status event.
        }
        #endregion

        #region ISpectator Members

        void ISpectator.WorkItemStateChanged(WorkItemStatus status)
        {
            Console.WriteLine(
                "Worker: {0}| WorkItem: {1}| State: {2}",
                status.WorkerId,
                status.WorkItemId,
                status.NewState);
        }

        #endregion

        private void LogWorkerStatusChange(WorkerStatus status)
        {
            Console.WriteLine(
                "Worker {0}| State: {1}",
                status.WorkerId,
                status.State);
        }
    }
}